package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.user.client.UserClient;
import com.chelvc.cloud.user.client.dto.UserDTO;
import com.chelvc.cloud.vehicle.client.dto.AssessRecoverDto;
import com.chelvc.cloud.vehicle.client.dto.GoodsAssessDto;
import com.chelvc.cloud.vehicle.client.param.AssessModifyParam;
import com.chelvc.cloud.vehicle.client.param.AssessQueryParam;
import com.chelvc.cloud.vehicle.client.param.AssessRecoverModifyParam;
import com.chelvc.cloud.vehicle.server.copier.AssessRecoverCopier;
import com.chelvc.cloud.vehicle.server.copier.GoodsAssessCopier;
import com.chelvc.cloud.vehicle.server.dao.AssessRecoverMapper;
import com.chelvc.cloud.vehicle.server.dao.AssessScoreMapper;
import com.chelvc.cloud.vehicle.server.dao.GoodsAssessMapper;
import com.chelvc.cloud.vehicle.server.entity.AssessRecover;
import com.chelvc.cloud.vehicle.server.entity.AssessScore;
import com.chelvc.cloud.vehicle.server.entity.GoodsAssess;
import com.chelvc.cloud.vehicle.server.service.AssessService;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author valley
 * @date 2024/01/30 00:03
 **/
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AssessServiceImpl extends ServiceImpl<GoodsAssessMapper, GoodsAssess> implements AssessService {

    private final UserClient userService;

    private static final SimpleDateFormat SDF = new SimpleDateFormat("yyyy-MM-dd");

    private final AssessRecoverMapper assessRecoverMapper;
    private final AssessScoreMapper assessScoreMapper;

    @Override
    public Long add(AssessModifyParam param) {
        Session user = SessionContextHolder.getSession();
        AssertUtils.available(this.getOne(new QueryWrapper<GoodsAssess>().eq("order_id", param.getOrderId()).eq(
                "creator", user.getId())) == null, "该订单已评价");
        GoodsAssess goodsAssess = GoodsAssessCopier.INSTANCE.copying(param);
        goodsAssess.setHits(0L);
        goodsAssess.setIsRecover(0);
        this.save(goodsAssess);
        baseMapper.updatePersonalAssessStatus(param.getOrderId(), param.getGoodsId());
        return goodsAssess.getId();
    }

    @Override
    public List<GoodsAssessDto> query(AssessQueryParam param) {
        QueryWrapper<GoodsAssess> wrapper = new QueryWrapper<>();
        //查询评价内容详情  单条查询时 增加点击数
        if (param.getId() != null) {
            wrapper.eq("id", param.getId());
        }
        //查询商品所有评价内容
        if (param.getGoodsId() != null) {
            wrapper.eq("goods_id", param.getGoodsId());
        }
        //查询个人订单评价内容
        if (param.getOrderId() != null) {
            wrapper.eq("order_id", param.getOrderId());
        }
        //按评价级别查询
        if (param.getScore() != null) {
            switch (param.getScore()) {
                case "1":
                    wrapper.ge("score", 0);
                    wrapper.lt("score", 2);
                    break;
                case "2":
                    wrapper.ge("score", 2);
                    wrapper.lt("score", 3);
                    break;
                case "3":
                    wrapper.ge("score", 3);
                    wrapper.lt("score", 4);
                    break;
                case "4":
                    wrapper.ge("score", 4);
                    wrapper.lt("score", 5);
                    break;
                case "5":
                    wrapper.eq("score", 5);
                    break;
                default:
                    break;
            }

        }
        //过滤是否回复
        if (param.getIsRecover() != null) {
            wrapper.eq("is_recover", param.getIsRecover());
        }

        if (param.getCreator() != null) {
            wrapper.eq("creator", param.getCreator());
        }

        //排序规则
        if (param.getCreator() == null) {
            wrapper.orderByDesc("hits", "score");
        } else {
            //查询个人时按照评论时间排序
            wrapper.orderByDesc("create_time");
        }
        List<GoodsAssess> goodsAssesses = this.list(wrapper);
        List<GoodsAssessDto> goodsAssesseDtos = GoodsAssessCopier.INSTANCE.copying(goodsAssesses);
        return goodsAssesseDtos.stream().peek(item -> {
            //个人信息获取
            UserDTO user = userService.getUser(item.getCreator());
            item.setCreatorAvatar(user.getAvatar());
            item.setCreatorNickName(user.getNickname());
            item.setCreatorTimeFormat(SDF.format(item.getCreateTime()));
            AssessRecover assessRecover = assessRecoverMapper.selectOne(new QueryWrapper<AssessRecover>().eq(
                    "assess_id", item.getId()));
            if (assessRecover != null) {
                AssessRecoverDto assessRecoverDto = AssessRecoverCopier.INSTANCE.copying(assessRecover);
                assessRecoverDto.setRecoverTimeFormat(SDF.format(assessRecoverDto.getRecoverTime()));
                item.setAssessRecoverDto(assessRecoverDto);
            }
            if (param.getId() != null) {
                GoodsAssess updateAssess = this.getById(item.getId());
                updateAssess.setHits(item.getHits() + 1);
                this.updateById(updateAssess);
            }

        }).collect(Collectors.toList());
    }

    @Override
    public Long addAssessRecover(AssessRecoverModifyParam param) {

        AssertUtils.available(assessRecoverMapper.selectOne(new QueryWrapper<AssessRecover>().eq("assess_id",
                param.getAssessId())) == null, "该订单已回复");
        AssessRecover assessRecover = AssessRecoverCopier.INSTANCE.copying(param);
        assessRecover.setRecoverTime(new Date());
        assessRecoverMapper.insert(assessRecover);
        GoodsAssess goodsAssess = this.getById(param.getAssessId());
        goodsAssess.setIsRecover(1);
        this.updateById(goodsAssess);
        return assessRecover.getId();

    }


    /**
     * 每天凌晨1点计算年、半年、季度、月、周商品平均分
     */
    @Scheduled(cron = "0 0 1 * * ?")
    @Override
    public void computeAvgScore() {
        Date date = new Date();
        this.baseMapper.clearScore();
        //获取所有评论， 按商品分类 分别计算 一年 半年 一个季度 一个月 一周 的评价分数
        List<Long> goodsIds = this.baseMapper.getAllGoodsIds();
        if (goodsIds == null) return;
        goodsIds.forEach(goodsId -> {
            List<GoodsAssess> goodsAssessList = this.list(new QueryWrapper<GoodsAssess>().eq("goods_id", goodsId));
            if (goodsAssessList.isEmpty()) return;
            BigDecimal goodsAvgScore = this.baseMapper.getAvgScore(goodsId).setScale(2, RoundingMode.HALF_UP);
            this.baseMapper.updateGoodsAvgScore(goodsAvgScore, goodsId);

            BigDecimal goodsYearScore = this.baseMapper.getGoodsScore(365, goodsId).setScale(2, RoundingMode.HALF_UP);
            BigDecimal goodsHalfYearScore = this.baseMapper.getGoodsScore(180, goodsId).setScale(2,
                    RoundingMode.HALF_UP);
            BigDecimal goodsQuarterScore = this.baseMapper.getGoodsScore(90, goodsId).setScale(2, RoundingMode.HALF_UP);
            BigDecimal goodsMonthScore = this.baseMapper.getGoodsScore(30, goodsId).setScale(2, RoundingMode.HALF_UP);
            BigDecimal goodsWeekScore = this.baseMapper.getGoodsScore(7, goodsId).setScale(2, RoundingMode.HALF_UP);
            AssessScore assessScore = new AssessScore();
            assessScore.setGoodsId(goodsId);
            assessScore.setYearScore(goodsYearScore);
            goodsHalfYearScore = goodsHalfYearScore.intValue() == 0 ? goodsYearScore : goodsHalfYearScore;
            assessScore.setHalfYearScore(goodsHalfYearScore);
            goodsQuarterScore = goodsQuarterScore.intValue() == 0 ? goodsHalfYearScore : goodsQuarterScore;
            assessScore.setQuarterScore(goodsQuarterScore);
            goodsMonthScore = goodsMonthScore.intValue() == 0 ? goodsQuarterScore : goodsMonthScore;
            assessScore.setMonthScore(goodsMonthScore);
            goodsWeekScore = goodsWeekScore.intValue() == 0 ? goodsMonthScore : goodsWeekScore;
            assessScore.setWeekScore(goodsWeekScore);
            assessScore.setModifyTime(date);
            this.assessScoreMapper.insert(assessScore);
        });
    }

    @Override
    public Map<String, BigDecimal> getGoodsScore(Long goodsId) {
        Map<String, BigDecimal> data = new HashMap<>();
        data.put("score", this.baseMapper.getGoodsAvgScore(goodsId).setScale(2, RoundingMode.HALF_UP));
        AssessScore assessScore = this.assessScoreMapper.selectOne(new QueryWrapper<AssessScore>().eq("goods_id",
                goodsId));
        data.put("yearScore", assessScore.getYearScore());
        data.put("halfYearScore", assessScore.getHalfYearScore());
        data.put("quarterScore", assessScore.getQuarterScore());
        data.put("monthScore", assessScore.getMonthScore());
        data.put("weekScore", assessScore.getWeekScore());
        return data;
    }

    @Override
    public void changeCommentInformFlag(Long userId) {
        LambdaUpdateWrapper<GoodsAssess> lwq = Wrappers.lambdaUpdate();
        lwq.eq(GoodsAssess::getCreator, userId);
        lwq.eq(GoodsAssess::getFlag, 0);
        lwq.set(GoodsAssess::getFlag, 1);
        baseMapper.update(null, lwq);
    }

    @Override
    public Integer queryNewInformNum(Long userId) {
        LambdaQueryWrapper<GoodsAssess> lwq = Wrappers.lambdaQuery();
        lwq.eq(GoodsAssess::getCreator, userId);
        lwq.eq(GoodsAssess::getFlag, 0);
        return baseMapper.selectCount(lwq);
    }

    @Override
    public Pagination<GoodsAssessDto> selectCommentListByUserId(Long userId, Paging paging) {
        Page<GoodsAssess> page = this.lambdaQuery()
                .eq(GoodsAssess::getCreator, userId)
                .orderByDesc(GoodsAssess::getCreateTime).page(DatabaseContextHolder.page(paging));
        List<GoodsAssess> goodsAssesses = page.getRecords();
        List<GoodsAssessDto> goodsAssesseDtos = GoodsAssessCopier.INSTANCE.copying(goodsAssesses);
        //个人信息获取
        UserDTO user = userService.getUser(userId);
        List<GoodsAssessDto> result = goodsAssesseDtos.stream().peek(item -> {
            item.setCreatorAvatar(user.getAvatar());
            item.setCreatorNickName(user.getNickname());
            item.setCreatorTimeFormat(SDF.format(item.getCreateTime()));
            AssessRecover assessRecover = assessRecoverMapper.selectOne(new QueryWrapper<AssessRecover>().eq(
                    "assess_id", item.getId()));
            if (assessRecover != null) {
                AssessRecoverDto assessRecoverDto = AssessRecoverCopier.INSTANCE.copying(assessRecover);
                assessRecoverDto.setRecoverTimeFormat(SDF.format(assessRecoverDto.getRecoverTime()));
                item.setAssessRecoverDto(assessRecoverDto);
            }
        }).collect(Collectors.toList());
        return DatabaseContextHolder.pagination(page, result);
    }

    @Override
    public Integer queryAssessStatus(Long orderId, Long goodsId, Long userId) {
        LambdaQueryWrapper<GoodsAssess> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(GoodsAssess::getOrderId, orderId);
        wrapper.eq(GoodsAssess::getGoodsId, goodsId);
        wrapper.eq(GoodsAssess::getCreator, userId);
        return baseMapper.selectCount(wrapper) > 0 ? 1 : 0;
    }
}
